import sqlobject
from thompcoutils.log_utils import get_logger


class DbUtils:
    def __init__(self, db_type, host, username, password, schema, port, sqlite_file=None):
        self.connection = None
        if db_type == "sqlite":
            if sqlite_file is None:
                raise RuntimeError("--sqlite requires --sqlite-dir")
            else:
                print("Using Sqlite database")
                self._connect_sqlite(sqlite_file)
        elif db_type == "postgres":
            self._connect_postgres(username, password, schema, host, port)
        elif db_type == "mysql":
            self._connect_mysql(username, password, schema, host, port)
        elif db_type == "odbc":
            raise RuntimeError("ODBC not implemented yet")
        else:
            raise RuntimeError("No database selected")

    def _connect_uri(self, uri):
        self.connection = sqlobject.sqlhub.processConnection = sqlobject.connectionForURI(uri)

    def _connect_sqlite(self, file_path):
        uri = "sqlite:" + file_path
        self._connect_uri(uri)

    def _connect_postgres(self, username, password, database, host, port):
        port_str = ""
        if port is not None:
            port_str = ":" + str(port)
        uri = "postgres://" + username + ":" + password + "@" + host + port_str + "/" + database
        self._connect_uri(uri)

    def _connect_mysql(self, username, password, database, host, port):
        port_str = ""
        if port is not None:
            port_str = ":" + str(port)
        uri = "mysql://" + username + ":" + password + "@" + host + port_str + "/" + database
        self._connect_uri(uri)

    def create_table(self, table):
        logger = get_logger()
        if not self.connection.tableExists(table.q.tableName):
            logger.debug("creating table {}".format(str(table)))
            table.createTable()

    def create_tables(self, tables):
        logger = get_logger()
        logger.debug("Creating tables...")
        for table in tables:
            self.create_table(table)

    @staticmethod
    def drop_tables(tables):
        logger = get_logger()
        logger.debug("Dropping tables...")
        for table in tables:
            logger.debug("dropping table {}".format(str(table)))
            table.dropTable(cascade=True, ifExists=True)



import os
from thompcoutils.log_utils import get_logger
import psutil
from sys import platform
import time
from netifaces import interfaces, ifaddresses, AF_INET


class UnhandledOs(Exception):
    pass


def _list_zones(walk_dir):
    d = {"timezones": []}
    files = os.listdir(walk_dir)
    for f in files:
        if os.path.isdir(os.path.join(walk_dir, f)):
            d[f] = _list_zones(os.path.join(walk_dir, f))
        else:
            d["timezones"].append(f)
    return d


def list_timezones():
    ost = os_type()
    if ost == "windows":
        time_zones = _list_zones("d:/temp/zoneinfo")
        time_zones["current_timezone"] = get_timezone()
        return time_zones
    elif ost == "linux" or ost == "osx":
        time_zones = _list_zones("/usr/share/zoneinfo")
        time_zones["current_timezone"] = get_timezone()
        return time_zones
    else:
        raise UnhandledOs("{} os not supported".format(ost))


def get_timezone():
    ost = os_type()
    if ost == "linux":
        with open("/etc/timezone") as f:
            return f.read().strip()
    elif ost == "windows" or ost == "osx":
        return time.tzname[0]
    else:
        raise UnhandledOs("{} os not supported".format(ost))


def kill_process(name):
    logger = get_logger()
    logger.info("killing process {}".format(name))
    ost = os_type()
    command = "sudo killall -9 {}".format(name)
    if ost == "linux":
        os.system(command)
    else:
        raise UnhandledOs("{} os not supported.  Linux command would look like {}".format(ost, command))


def is_running(name):
    pid_list = psutil.pids()
    for pid in pid_list:
        # noinspection PyBroadException
        try:
            process = psutil.Process(pid)
            if name == process.name():
                return True
        except Exception:
            pass
    return False


class OSType(enumerate):
    Linux = "Linux"
    OSX = "OSX"
    WINDOWS = "Windows"


def os_type():
    if platform == "linux" or platform == "linux2":
        return OSType.Linux
    elif platform == "darwin":
        return OSType.OSX
    elif platform == "win32":
        return OSType.WINDOWS
    else:
        raise UnhandledOs("Unknown system")


def get_ip_addresses():
    addresses = None
    for interface_name in interfaces():
        addresses = [i['addr'] for i in ifaddresses(interface_name).setdefault(AF_INET, [{'addr': 'No IP addr'}])]
    return addresses



import smtplib
import os
from email import encoders
import mimetypes
from email.mime.audio import MIMEAudio
from email.mime.base import MIMEBase
from email.mime.image import MIMEImage
from thompcoutils.config_utils import ConfigManager
from thompcoutils.log_utils import get_logger

if os.name == 'nt' or os.name == "posix":
    from email.mime.multipart import MIMEMultipart
    from email.mime.text import MIMEText
else:
    # noinspection PyUnresolvedReferences
    from email.MIMEMultipart import MIMEMultipart
    # noinspection PyUnresolvedReferences
    from email.MIMEText import MIMEText
    # noinspection PyUnresolvedReferences
    from email.MIMEApplication import MIMEApplication


class EmailConnection:
    TITLE = "email connection"
    USERNAME = "username"
    PASSWORD = "password"
    FROM = "from"
    SMTP = "smtp"
    PORT = "port"

    def __init__(self, cfg_mgr=None, username=None, password=None, from_user=None, smtp=None, port=None):
        if cfg_mgr is None:
            self.username = username
            self.password = password
            self.from_user = from_user
            self.smtp = smtp
            self.port = port
        else:
            self.username = cfg_mgr.read_entry(EmailConnection.TITLE, EmailConnection.USERNAME,
                                               "myname@google.com", str)
            self.password = cfg_mgr.read_entry(EmailConnection.TITLE, EmailConnection.PASSWORD,
                                               "mySecretPassword", str)
            self.from_user = cfg_mgr.read_entry(EmailConnection.TITLE, EmailConnection.FROM,
                                                "Email Sender", str)
            self.smtp = cfg_mgr.read_entry(EmailConnection.SMTP, EmailConnection.FROM,
                                           "smtp.gmail.com", str)
            self.port = cfg_mgr.read_entry(EmailConnection.PORT, EmailConnection.FROM,
                                           587, int)


class EmailSender:
    def __init__(self, email_connection):
        self.email_connection = email_connection

    def send(self, to_email, subject, message, attach_files=None):
        logger = get_logger()
        server = None
        try:
            server = smtplib.SMTP(self.email_connection.smtp, self.email_connection.port)
            server.ehlo()
            server.starttls()
            server.login(self.email_connection.username, self.email_connection.password)
            outer_msg = MIMEMultipart('alternative')
            sender = self.email_connection.from_user
            recipients = to_email
            outer_msg['Subject'] = subject
            outer_msg.attach(MIMEText(message, "html"))
            outer_msg['From'] = self.email_connection.from_user
            if isinstance(recipients, list):
                outer_msg['To'] = ", ".join(recipients)
            else:
                outer_msg['To'] = recipients
            if attach_files is not None:
                for filename in attach_files:
                    file_type, encoding = mimetypes.guess_type(filename)
                    if file_type is None or encoding is not None:
                        file_type = 'application/octet-stream'
                    maintype, subtype = file_type.split('/', 1)
                    if maintype == 'text':
                        with open(filename) as fp:
                            msg = MIMEText(fp.read(), _subtype=subtype)
                    elif maintype == 'image':
                        with open(filename, 'rb') as fp:
                            msg = MIMEImage(fp.read(), _subtype=subtype)
                    elif maintype == 'audio':
                        with open(filename, 'rb') as fp:
                            msg = MIMEAudio(fp.read(), _subtype=subtype)
                    else:
                        with open(filename, 'rb') as fp:
                            msg = MIMEBase(maintype, subtype)
                            msg.set_payload(fp.read())
                        encoders.encode_base64(msg)
                    msg.add_header('Content-Disposition', 'attachment', filename=filename)
                    outer_msg.attach(msg)
            server.sendmail(sender, recipients, outer_msg.as_string())
            logger.debug("Successfully sent mail to " + str(recipients))
        except Exception as e:
            logger.warning("Failed to send mail to {} because {}".format(to_email, str(e)))
        finally:
            if server is not None:
                server.quit()


def main():
    send_file = True
    use_parameters = False
    if use_parameters:
        mail = EmailSender(EmailConnection(username="jorythompson@gmail.com",
                                           password="warfzdvvcovseosv",
                                           from_user="Test",
                                           smtp="smtp.gmail.com",
                                           port=587))
        subject = "using parameters"
    else:
        config = ConfigManager("email.ini")
        mail = EmailSender(EmailConnection(config))
        subject = "using configuration file"

    if send_file:
        mail.send(to_email="Jordan@ThompCo.com", subject="this is a test from emailSender",
                  message="Here is the message using parameters passed in with an attached file",
                  attach_files=[os.path.basename(__file__)])
    else:
        mail.send("jorythompson@gmail.com", "this is a test from emailSender {}".format(subject),
                  "This is a test message")


if __name__ == "__main__":
    main()



import sys
from configparser import ConfigParser, NoOptionError, NoSectionError
from thompcoutils.log_utils import get_logger
import os


class ConfigManager:
    def __init__(self, file_name, title=None, create=False):
        self.file_name = file_name
        self.config = ConfigParser()
        self.config.optionxform = str
        self.create = create
        if not create:
            if os.path.exists(file_name):
                self.config.read(file_name)
            else:
                raise Exception("File {} does not exist!".format(file_name))
        self.notes = []
        self.title = title
        self.values = {}

    @staticmethod
    def missing_entry(section, entry, file_name, default_value=None):
        logger = get_logger()
        logger.debug("starting")
        if default_value is None:
            log_fn = logger.critical
            message = "Required entry"
            default_value = ""
        else:
            log_fn = logger.debug
            message = "Entry"
            if default_value == "":
                default_value = "Ignoring."
            else:
                default_value = "Using default value of (" + str(default_value) + ")"
        log_fn(message + " \"" + entry + "\" in section [" + section + "] in file: " + file_name
               + " is malformed or missing.  " + str(default_value))
        if default_value == "":
            log_fn("Exiting now")
            sys.exit()

    @staticmethod
    def _insert_note(lines, line_number, note):
        if "\n" in note:
            message = note.split("\n")
        else:
            message = note
        if type(message) == str:
            lines.insert(line_number, "# " + message + ":\n")
        else:
            for l in message[:-1]:
                lines.insert(line_number, "# " + l + "\n")
                line_number += 1
            lines.insert(line_number, "# " + message[-1] + ":\n")

    def read_entry(self, section, entry, default_value, notes=None):
        value = default_value
        if self.create:
            # noinspection PyBroadException
            try:
                self.config.add_section(section)
            except Exception:
                pass
            if notes is not None:
                self.notes.append({"section": section,
                                   "entry": entry,
                                   "notes": notes})
            self.config.set(section, entry, str(default_value))
        else:
            try:
                if type(default_value) is str:
                    value = self.config.get(section, entry)
                elif type(default_value) is int:
                    value = self.config.getint(section, entry)
                elif type(default_value) is float:
                    value = self.config.getfloat(section, entry)
                elif type(default_value) is bool:
                    value = self.config.getboolean(section, entry)
                else:
                    print("type not handled for ()".format(default_value))
            except (NoSectionError, NoOptionError):
                ConfigManager.missing_entry(section, entry, self.file_name, default_value)
        try:
            self.values[section][entry] = value
        except:
            self.values[section] = {}
            self.values[section][entry] = value
        return value

    def read_section(self, section, default_entries, notes=None):
        key_values = default_entries
        if self.create:
            # noinspection PyBroadException
            try:
                self.config.add_section(section)
            except Exception:
                pass
            for entry in default_entries:
                self.config.set(section, str(entry), str(default_entries[entry]))
            if notes is not None:
                self.notes.append({"section": section,
                                   "entry": None,
                                   "notes": notes})
        else:
            key_values = dict()
            for (key, val) in self.config.items(section):
                key_values[key] = val
        return key_values

    def write(self, out_file):
        if os.path.isfile(out_file):
            print("File {} exists!  You must remove it before running this".format(out_file))
            # sys.exit()
        f = open(out_file, "w")
        self.config.write(f)
        f.close()
        f = open(out_file)
        lines = f.readlines()
        f.close()
        if self.title is not None:
            ConfigManager._insert_note(lines, 0, self.title)
        for note in self.notes:
            in_section = False
            line_number = 0
            for line in lines:
                if "[" + note["section"] + "]" in line:
                    if note["entry"] is None:
                        ConfigManager._insert_note(lines, line_number, note["notes"])
                        break
                    else:
                        in_section = True
                elif line.startswith("[") and line.endswith("]"):
                    in_section = False
                if in_section:
                    if line.startswith(note["entry"]):
                        ConfigManager._insert_note(lines, line_number, note["notes"])
                        break
                line_number += 1
        f = open(out_file, "w")
        contents = "".join(lines)
        f.write(contents)
        f.close()
        print("Done writing {}".format(out_file))
        sys.exit()


def main():
    write = True  # set this to True to create the ini file
    cfg_mgr = ConfigManager("test.ini", "This is the title of the ini file\n"
                                        "You can have multiple lines if you use line breaks", write)
    first = cfg_mgr.read_entry("User 1", "first name", "Joe", "This is the first name")
    last = cfg_mgr.read_entry("User 1", "last name", "Brown", "This is the last name")
    age = cfg_mgr.read_entry("User 1", "age", 12)
    is_male = cfg_mgr.read_entry("User 1", "male", True)
    weight = cfg_mgr.read_entry("User 1", "weight", 23.5)
    section = cfg_mgr.read_section("user 2", {"first name": "Sally",
                                              "last name": "Jones",
                                              "age": 15,
                                              "is_male": False,
                                              "weight": 41.3},
                                   "You only get to add notes at the top of the section using this method")

    print(first)
    print(last)
    print(age)
    print(is_male)
    print(weight)
    print(section)

    if write:
        cfg_mgr.write("another.ini")


if __name__ == "__main__":
    main()
